// ChipBrowser.cpp : Defines the entry point for the console application.
// 8085 CPU Simulator
// Version: 1.0
// Author: Pavel Zima
// Date: 1.1.2013

#include <tchar.h>

#include <Windows.h>
#include <locale.h>
#include <GdiPlus.h>
#include <vector>

using namespace Gdiplus;
using namespace std;

#pragma comment (lib, "gdiplus.lib")

// it says that fopen is unsafe
#pragma warning(disable : 4996)
// we need really big stack for recursive flood fill (need to be reimplemented)
#pragma comment(linker, "/STACK:536870912")

#define DIVISOR 1000 // the lower the faster is clock, 1000 is lowest value I achieved
#define MINSHAPESIZE 50 // if a shape is smaller than this it gets reported

#define VCC 5.0f
#define QUANTUM 35.0f // basic quantum of charge moved in one cycle
#define MAXQUANTUM 150.0f // charge gets truncated to this value
#define PULLUPDEFLATOR 2.0f // positive charge gets divided by this

#define METAL 1
#define POLYSILICON 2
#define DIFFUSION 4
#define VIAS 8
#define BURIED 16 // layer
#define PADS 32
#define TRANSISTORS 64
#define VIAS_TO_POLYSILICON 128
#define VIAS_TO_DIFFUSION 256
#define BURIED_CONTACT 512 // real contact
#define REAL_DIFFUSION 1024 // i.e. no transistor
#define TEMPORARY 32768

// basic signals - they get their own color while exporting figures

#define GND 1
#define SIG_VCC 2

// definition of pads

#define PAD_D0 3
#define PAD_D1 4
#define PAD_D2 5
#define PAD_D3 6
#define PAD_D4 7
#define PAD_D5 8
#define PAD_D6 9
#define PAD_D7 10

#define PAD_R_W 11

#define PAD_CS0 12
#define PAD_CS1 13
#define PAD_CS2 14
#define PAD_CS3 15
#define PAD__CS4 16
#define PAD__CS5 17
#define PAD__CS6 18

#define PAD__RST 19
#define PAD_PHI2 20

// all other signals are numbered automaticaly

#define FIRST_SIGNAL 21

int size_x, size_y;
unsigned __int16 *pombuf;
unsigned __int16 *signals_metal;
unsigned __int16 *signals_poly;
unsigned __int16 *signals_diff;

// trying to keep FillObject local heap as small as possible
int type, type2, type3;
BitmapData *bd;
int shapesize;
bool objectfound;
int nextsignal;

#define GATE 1
#define DRAIN 2
#define SOURCE 3

// connection to transistor remembers index of connected transistor and its terminal
// proportion is the proportion of that transisor are to the area of all transistors connected
// to the respective signal - it is here for optimalisation purposes

class connection
{
public:
	connection();
	int terminal;
	int index;
	float proportion;
};

connection::connection()
{
	terminal = index = 0;
	proportion = 0.0f;
}

#define SIG_GND 1
#define SIG_PWR 2
#define SIG_FLOATING 3

// signal keeps the vector of connections (i.e. all transistors connected to the respective signal)
// Homogenize() averages the charge proportionally by transistor area
// ignore means that this signal need not to be homogenized - a try for optimalization
// but it works only for Vcc and GND

class signal
{
public:
	signal();
	void Homogenize();
	vector<connection> connections;
	float signalarea;
	bool ignore;
};

signal::signal()
{
	signalarea = 0.0f;
	ignore = false;
}

vector<signal> signals;

#define PAD_INPUT 1
#define PAD_OUTPUT 2
#define PAD_BIDIRECTIONAL 3

// PADs - used for communication with the CPU see its use in simulation below
class pad
{
public:
	pad();
	void SetInputSignal(int signal);
	float ReadOutput();
	int ReadOutputStatus();
	int ReadInputStatus();
	int type;
	int x, y;
	int origsignal;
	vector<connection> connections;
};

pad::pad()
{
	type = 0;
	x = y = 0;
	origsignal = 0;
}

vector<pad> pads;

// transistor - keeps connections to other transistors
// Simulate() - moves charge between source and drain
class transistor
{
public:
	transistor();
	bool IsOn();
	int IsOnAnalog();
	void Simulate();
	void Normalize();
	int Valuate();

	int x, y;
	int gate, source, drain;
	int sourcelen, drainlen, otherlen;
	float area;
	bool depletion;

	float resist;
	float gatecharge, sourcecharge, draincharge;

	vector<connection> gateconnections;
	vector<connection> sourceconnections;
	vector<connection> drainconnections;
	float gateneighborhood, sourceneighborhood, drainneighborhood;
	float chargetobeon, pomchargetogo;
};

transistor::transistor()
{
	x = y = 0;
	gate = source = drain = 0;
	sourcelen = drainlen = otherlen = 0;
	area = 0.0f;
	depletion = false;
	resist = 0.0f;
	gatecharge = sourcecharge = draincharge = 0.0f;
	gateneighborhood = sourceneighborhood = drainneighborhood = 0.0f;
	chargetobeon = pomchargetogo = 0.0f;
}

inline bool transistor::IsOn()
{
	if (gatecharge > 0.0f)
		return true;
	return false;
}

int transistor::IsOnAnalog()
{
	return int(50.0f * gatecharge / area) + 50;
}

// Gets the type of the transistor - originally for optimalization purposes now more for statistical purposes
inline int transistor::Valuate()
{
	// 1 depletion pullup
	// 2 enhancement pullup
	// 3 direct pulldown
	// 4 other

	if (depletion)
		return 1;
	if (drain == SIG_VCC)
		return 2;
	if (source == SIG_GND)
		return 3;
	return 4;
}

vector<transistor> transistors;

void signal::Homogenize()
{
	float pomcharge = 0.0f;
	for (unsigned int i = 0; i < connections.size(); i++)
	{
		if (connections[i].terminal == GATE)
			pomcharge += transistors[connections[i].index].gatecharge;
		else if (connections[i].terminal == SOURCE)
			pomcharge += transistors[connections[i].index].sourcecharge;
		else if (connections[i].terminal == DRAIN)
			pomcharge += transistors[connections[i].index].draincharge;
	}

	for (unsigned int i = 0; i < connections.size(); i++)
	{
		if (connections[i].terminal == GATE)
			transistors[connections[i].index].gatecharge = pomcharge * connections[i].proportion;
		else if (connections[i].terminal == SOURCE)
			transistors[connections[i].index].sourcecharge = pomcharge * connections[i].proportion;
		else if (connections[i].terminal == DRAIN)
			transistors[connections[i].index].draincharge = pomcharge * connections[i].proportion;
	}
}

void transistor::Simulate()
{
	if (gate == SIG_GND)
		gatecharge = 0.0f;
	else if (gate == SIG_VCC)
		gatecharge = area;

	if (depletion)
	{
		float chargetogo = pomchargetogo;

		chargetogo /= PULLUPDEFLATOR; // pull-ups are too strong, we need to weaken them

		for (unsigned int i = 0; i < sourceconnections.size(); i++)
		{
			if (sourceconnections[i].terminal == GATE)
				transistors[sourceconnections[i].index].gatecharge += chargetogo * sourceconnections[i].proportion;
			else if (sourceconnections[i].terminal == SOURCE)
				transistors[sourceconnections[i].index].sourcecharge += chargetogo * sourceconnections[i].proportion;
			else if (sourceconnections[i].terminal == DRAIN)
				transistors[sourceconnections[i].index].draincharge += chargetogo * sourceconnections[i].proportion;
		}
	}
	else
	{
		if (IsOn())
		{
			if ((drain == SIG_VCC))
			{
				float chargetogo = pomchargetogo;
				chargetogo *= gatecharge / area;
				chargetogo /= PULLUPDEFLATOR;

				for (unsigned int i = 0; i < sourceconnections.size(); i++)
				{
					if (sourceconnections[i].terminal == GATE)
						transistors[sourceconnections[i].index].gatecharge += chargetogo * sourceconnections[i].proportion;
					else if (sourceconnections[i].terminal == SOURCE)
						transistors[sourceconnections[i].index].sourcecharge += chargetogo * sourceconnections[i].proportion;
					else if (sourceconnections[i].terminal == DRAIN)
						transistors[sourceconnections[i].index].draincharge += chargetogo * sourceconnections[i].proportion;
				}
			}
			else if ((source == SIG_GND))
			{
				float chargetogo = pomchargetogo;
				chargetogo *= gatecharge / area;

				for (unsigned int i = 0; i < drainconnections.size(); i++)
				{
					if (drainconnections[i].terminal == GATE)
						transistors[drainconnections[i].index].gatecharge -= chargetogo * drainconnections[i].proportion;
					else if (drainconnections[i].terminal == SOURCE)
						transistors[drainconnections[i].index].sourcecharge -= chargetogo * drainconnections[i].proportion;
					else if (drainconnections[i].terminal == DRAIN)
						transistors[drainconnections[i].index].draincharge -= chargetogo * drainconnections[i].proportion;
				}
			}
			else
			{
				float pomsourcecharge = 0.0f, pomdraincharge = 0.0f;

				pomsourcecharge = sourcecharge;
				if (pomsourcecharge > 0.0f)
					pomsourcecharge /= PULLUPDEFLATOR;

				pomdraincharge = draincharge;
				if (pomdraincharge > 0.0f)
					pomdraincharge /= PULLUPDEFLATOR;

				float chargetogo = ((pomsourcecharge - pomdraincharge) / resist) / PULLUPDEFLATOR;
				float pomsign = 1.0;
				if (chargetogo < 0.0f)
				{
					pomsign = -1.0;
					chargetogo = -chargetogo;
				}
				if (chargetogo > MAXQUANTUM)
					chargetogo = MAXQUANTUM;
				chargetogo *= gatecharge / area;
				chargetogo *= pomsign;

				sourcecharge -= chargetogo;
				draincharge += chargetogo;
			}
		}
	}
}

void transistor::Normalize()
{
	if (gatecharge < -area)
		gatecharge = -area;
	if (gatecharge > area)
		gatecharge = area;
	if (sourcecharge < -area)
		sourcecharge = -area;
	if (sourcecharge > area)
		sourcecharge = area;
	if (draincharge < -area)
		draincharge = -area;
	if (draincharge > area)
		draincharge = area;
}

int pad::ReadInputStatus()
{
	int pomvalue = SIG_FLOATING;

	if (connections.size())
	{
		if (connections[0].terminal == GATE)
			pomvalue = transistors[connections[0].index].gate;
		else if (connections[0].terminal == SOURCE)
			pomvalue = transistors[connections[0].index].source;
		else if (connections[0].terminal == DRAIN)
			pomvalue = transistors[connections[0].index].drain;
	}

	if (pomvalue > SIG_VCC)
		pomvalue = SIG_FLOATING;

	return pomvalue;
}

void pad::SetInputSignal(int signal)
{
	for (unsigned int i = 0; i < connections.size(); i++)
	{
		if (signal != SIG_FLOATING)
		{
			if (connections[i].terminal == GATE)
				transistors[connections[i].index].gate = signal;
			else if (connections[i].terminal == SOURCE)
				transistors[connections[i].index].source = signal;
			else if (connections[i].terminal == DRAIN)
				transistors[connections[i].index].drain = signal;
		}
		else
		{
			if (connections[i].terminal == GATE)
				transistors[connections[i].index].gate = origsignal;
			else if (connections[i].terminal == SOURCE)
				transistors[connections[i].index].source = origsignal;
			else if (connections[i].terminal == DRAIN)
				transistors[connections[i].index].drain = origsignal;
		}
	}
}

float pad::ReadOutput()
{
	float shouldbe = 0.0f, reallywas = 0.0f;
	for (unsigned int i = 0; i < connections.size(); i++)
	{
		if (connections[i].terminal == SOURCE)
		{
			shouldbe += transistors[connections[i].index].area;
			reallywas += transistors[connections[i].index].sourcecharge;
		}
		else if (connections[i].terminal == DRAIN)
		{
			shouldbe += transistors[connections[i].index].area;
			reallywas += transistors[connections[i].index].draincharge;
		}
	}

	return reallywas / shouldbe;
}

// does not work correctly
// i.e. it cannot recoginze floating status
int pad::ReadOutputStatus()
{
	float pom = ReadOutput();
	if (pom < -0.05f)
		return SIG_GND;
	else if (pom > 0.05f)
		return SIG_VCC;
	return SIG_FLOATING;
}

// This is a helper function for Windows API purposes as we need to work with PNG files
int GetEncoderClsid(const wchar_t* format, CLSID *pClsid)
{
	UINT pocet = 0;
	UINT velikost = 0;
	ImageCodecInfo *pImageCodecInfo = NULL;

	GetImageEncodersSize(&pocet, &velikost);
	if (!velikost)
		return -1;
	pImageCodecInfo = (ImageCodecInfo *) (malloc(velikost));
	if (!pImageCodecInfo)
		return -1;

	GetImageEncoders(pocet, velikost, pImageCodecInfo);
	for (UINT i = 0; i < pocet; i++)
	{
		if (wcscmp(pImageCodecInfo[i].MimeType, format) == 0)
		{
			*pClsid = pImageCodecInfo[i].Clsid;
			free(pImageCodecInfo);
			return i;
		}
	}
	free(pImageCodecInfo);
	return -1;
}

int GetPixelFromBitmapData(BitmapData *bd, int x, int y)
{
	if (x < 0)
		return 0;
	if (y < 0)
		return 0;
	if (x >= size_x)
		return 0;
	if (y >= size_y)
		return 0;

	BYTE *bajt = (BYTE *) bd->Scan0 + y * bd->Stride + 3 * x;
	int barva = *bajt++;
	barva <<= 8;
	barva += *bajt++;
	barva <<= 8;
	barva += *bajt;

	return barva;
}

bool SetPixelToBitmapData(BitmapData *bd, int x, int y, int barva)
{
	if (x < 0)
		return false;
	if (y < 0)
		return false;
	if (x >= size_x)
		return false;
	if (y >= size_y)
		return false;

	BYTE *bajt = (BYTE *) bd->Scan0 + y * bd->Stride + 4 * x;
	*bajt++ = barva & 0xff;
	*bajt++ = (barva & 0xff00) >> 8;
	*bajt++ = (barva & 0xff0000) >> 16;
	if (barva)
		*bajt = 0xff;
	else
		*bajt = 0x00;

	return true;
}

// These functions are very poorly implemented - they use recursive flood algorithm which is very slow and time consuming
// should be reimplemented

bool FillObject(int x, int y)
{
	if (pombuf[y * size_x + x] & type)
		return false;
	if (!(GetPixelFromBitmapData(bd, x, y)))
		return false;
	pombuf[y * size_x + x] |= type;
	shapesize++;

	if (x)
		if (!(pombuf[y * size_x + x - 1] & type))
			FillObject(x - 1, y);
	if (y)
		if (!(pombuf[(y - 1) * size_x + x] & type))
			FillObject(x, y - 1);
	if (x < size_x - 1)
		if (!(pombuf[y * size_x + x + 1] & type))
			FillObject(x + 1, y);
	if (y < size_y - 1)
		if (!(pombuf[(y + 1) * size_x + x] & type))
			FillObject(x, y + 1);

	return true;
}

bool FillStructure(int x, int y)
{
	if (pombuf[y * size_x + x] & type)
		return false;
	if ((pombuf[y * size_x + x] & type2) != type3)
		return false;
	pombuf[y * size_x + x] |= type;
	shapesize++;

	if (x)
		FillStructure(x - 1, y);
	if (y)
		FillStructure(x, y - 1);
	if (x < size_x - 1)
		FillStructure(x + 1, y);
	if (y < size_y - 1)
		FillStructure(x, y + 1);

	return true;
}

bool FillStructureCheck(int x, int y)
{
	if (pombuf[y * size_x + x] & type)
		return false;
	if ((pombuf[y * size_x + x] & type2) != type2)
		return false;
	pombuf[y * size_x + x] |= type;

	if ((pombuf[y * size_x + x] & type3))
		objectfound = true;

	if (x)
		FillStructureCheck(x - 1, y);
	if (y)
		FillStructureCheck(x, y - 1);
	if (x < size_x - 1)
		FillStructureCheck(x + 1, y);
	if (y < size_y - 1)
		FillStructureCheck(x, y + 1);

	return true;
}

void ClearTemporary()
{
	for (int y = 0; y < size_y; y++)
		for (int x = 0; x < size_x; x++)
			pombuf[y * size_x + x] &= ~TEMPORARY;
}

// opens the respective file and copies its content to pombuff which holds the layers
void CheckFile(TCHAR *firstpart, int ltype)
{
	type = ltype;

	TCHAR filename[256];
	if (type == VIAS)
		wsprintf(filename, L"%s%s", firstpart, L"_vias.png");
	if (type == METAL)
		wsprintf(filename, L"%s%s", firstpart, L"_metal.png");
	if (type == PADS)
		wsprintf(filename, L"%s%s", firstpart, L"_pads.png");
	if (type == POLYSILICON)
		wsprintf(filename, L"%s%s", firstpart, L"_polysilicon.png");
	if (type == DIFFUSION)
		wsprintf(filename, L"%s%s", firstpart, L"_diffusion.png");
	if (type == BURIED)
		wsprintf(filename, L"%s%s", firstpart, L"_buried.png");
	wprintf(filename);

	Bitmap *bitmapa;
	bitmapa = new Bitmap(filename);
	if (bitmapa->GetLastStatus() != Ok)
	{
		wprintf(L"\nCannot open file: %s\n", filename);
		delete bitmapa;
		return;
	}

	size_x = bitmapa->GetWidth();
	size_y = bitmapa->GetHeight();

	wprintf(L" size x: %d, y: %d\n", size_x, size_y);

	if (!pombuf)
	{
		pombuf = new unsigned __int16[size_x * size_y];
		ZeroMemory(&pombuf[0], size_x * size_y * sizeof(unsigned __int16));
	}

	BitmapData bitmapdata;
	bitmapa->LockBits(&Rect(0, 0, size_x, size_y), ImageLockModeRead, PixelFormat24bppRGB, &bitmapdata);
	wprintf(L"Bitmap x: %d, y: %d, stride: %d\n", bitmapdata.Width, bitmapdata.Height, bitmapdata.Stride);
	bd = &bitmapdata;

	int bitmap_room = 0;
	for (int y = 0; y < size_y; y++)
		for (int x = 0; x < size_x; x++)
			if (GetPixelFromBitmapData(&bitmapdata, x, y))
				bitmap_room++;
	wprintf(L"Percentage: %.2f%%\n", 100.0 * bitmap_room / size_x / size_y);

	int bitmap_count = 0;
	for (int y = 0; y < size_y; y++)
		for (int x = 0; x < size_x; x++)
			if (shapesize = 0, FillObject(x, y))
			{
				bitmap_count++;
				if (shapesize < MINSHAPESIZE)
					wprintf(L"Object at %d, %d is too small.\n", x, y);
			}
	wprintf(L"Count: %d\n", bitmap_count);
	wprintf(L"---------------------\n");

	bitmapa->UnlockBits(&bitmapdata);
	delete bitmapa;
}

// routes the signal thru all the layers - necessary for numbering the signals
bool RouteSignal(int x, int y, int sig_num, int layer)
{
	switch (layer)
	{
		case METAL:
			if (!(pombuf[y * size_x + x] & METAL))
				return false;
			break;
		case POLYSILICON:
			if (!(pombuf[y * size_x + x] & POLYSILICON))
				return false;
			break;
		case DIFFUSION:
			if (!(pombuf[y * size_x + x] & REAL_DIFFUSION))
				return false;
			break;
	}

	int pomsig = 0;
	switch (layer)
	{
		case METAL:
			pomsig = signals_metal[y * size_x + x];
			break;
		case POLYSILICON:
			pomsig = signals_poly[y * size_x + x];
			break;
		case DIFFUSION:
			pomsig = signals_diff[y * size_x + x];
			break;
	}
	if (pomsig)
	{
		if (pomsig != sig_num)
			wprintf(L"Signal mismatch %d vs %d at %d, %d\n", sig_num, pomsig, x, y);
		return false;
	}

	switch (layer)
	{
		case METAL:
			signals_metal[y * size_x + x] = sig_num;
			break;
		case POLYSILICON:
			signals_poly[y * size_x + x] = sig_num;
			break;
		case DIFFUSION:
			signals_diff[y * size_x + x] = sig_num;
			break;
	}

	if (x)
		RouteSignal(x - 1, y, sig_num, layer);
	if (y)
		RouteSignal(x, y - 1, sig_num, layer);
	if (x < size_x - 1)
		RouteSignal(x + 1, y, sig_num, layer);
	if (y < size_y - 1)
		RouteSignal(x, y + 1, sig_num, layer);

	switch (layer)
	{
		case METAL:
			if ((pombuf[y * size_x + x] & VIAS_TO_POLYSILICON))
				RouteSignal(x, y, sig_num, POLYSILICON);
			else if ((pombuf[y * size_x + x] & VIAS_TO_DIFFUSION))
				RouteSignal(x, y, sig_num, DIFFUSION);
			break;
		case POLYSILICON:
			if ((pombuf[y * size_x + x] & VIAS_TO_POLYSILICON))
				RouteSignal(x, y, sig_num, METAL);
			else if ((pombuf[y * size_x + x] & BURIED_CONTACT))
				RouteSignal(x, y, sig_num, DIFFUSION);
			break;
		case DIFFUSION:
			if ((pombuf[y * size_x + x] & VIAS_TO_DIFFUSION))
				RouteSignal(x, y, sig_num, METAL);
			else if ((pombuf[y * size_x + x] & BURIED_CONTACT))
				RouteSignal(x, y, sig_num, POLYSILICON);
			break;
	}

	return true;
}

int gate, source, drain;
int sourcelen, drainlen, otherlen;

// helping function for setting source and drain of the transistor
void SetSourceDran(int x, int y, int value)
{
	if (!source)
	{
		source = value;
		sourcelen++;
	}
	else
	{
		if (source == value)
		{
			sourcelen++;
		}
		else
		{
			if (!drain)
			{
				drain = value;
				drainlen++;
			}
			else
			{
				if (drain == value)
				{
					drainlen++;
				}
				else
				{
					wprintf(L"Transistor has more than 3 terminals at %d %d.\n", x, y);
				}
			}
		}
	}
}

// gets the value of 8 transistors - for listing purposes
int GetRegVal(unsigned int reg[])
{
	int pomvalue = 0;
	for (int i = 7; i >= 0; i--)
		pomvalue = pomvalue << 1 | transistors[reg[i]].IsOn() & 1;
	return pomvalue;
}

void WriteTransCoords(int bit7, int bit6, int bit5, int bit4, int bit3, int bit2, int bit1, int bit0)
{
	wprintf(L"[%d, %d], ", transistors[bit7].x, transistors[bit7].y);
	wprintf(L"[%d, %d], ", transistors[bit6].x, transistors[bit6].y);
	wprintf(L"[%d, %d], ", transistors[bit5].x, transistors[bit5].y);
	wprintf(L"[%d, %d], ", transistors[bit4].x, transistors[bit4].y);
	wprintf(L"[%d, %d], ", transistors[bit3].x, transistors[bit3].y);
	wprintf(L"[%d, %d], ", transistors[bit2].x, transistors[bit2].y);
	wprintf(L"[%d, %d], ", transistors[bit1].x, transistors[bit1].y);
	wprintf(L"[%d, %d]", transistors[bit0].x, transistors[bit0].y);
}

// finds the transistor by coordinates - the coordinations must be upper - left corner ie the most top (first) and most left (second) corner
int FindTransistor(unsigned int x, unsigned int y)
{
	for (unsigned int i = 0; i < transistors.size(); i++)
		if (transistors[i].x == x && transistors[i].y == y)
			return i;
	wprintf(L"--- Error --- Transistor at %d, %d not found.\n", x, y);
	return -1;
}

void CheckTransistor(int x, int y)
{
	if ((pombuf[y * size_x + x] & (TRANSISTORS | TEMPORARY)) != TRANSISTORS)
		return;

	if (!signals_poly[y * size_x + x])
		wprintf(L"Transistor with no signal in gate at: %d %d.\n", x, y);
	if (!gate)
		gate = signals_poly[y * size_x + x];
	if (gate != signals_poly[y * size_x + x])
		wprintf(L"Ambiguous signals in poly for transistor at: %d %d.\n", x, y);

	pombuf[y * size_x + x] |= TEMPORARY;
	shapesize++;

	if (x)
	{
		if ((pombuf[y * size_x + x - 1] & TRANSISTORS))
		{
			CheckTransistor(x - 1, y);
		}
		else
		{
			if ((pombuf[y * size_x + x - 1] & DIFFUSION))
				SetSourceDran(x, y, signals_diff[y * size_x + x - 1]);
			else
				otherlen++;
		}
	}
	if (y)
	{
		if ((pombuf[(y - 1) * size_x + x] & TRANSISTORS))
		{
			CheckTransistor(x, y - 1);
		}
		else
		{
			if ((pombuf[(y - 1) * size_x + x] & DIFFUSION))
				SetSourceDran(x, y, signals_diff[(y - 1) * size_x + x]);
			else
				otherlen++;
		}
	}
	if (x < size_x - 1)
	{
		if ((pombuf[y * size_x + x + 1] & TRANSISTORS))
		{
			CheckTransistor(x + 1, y);
		}
		else
		{
			if ((pombuf[y * size_x + x + 1] & DIFFUSION))
				SetSourceDran(x, y, signals_diff[y * size_x + x + 1]);
			else
				otherlen++;
		}
	}
	if (y < size_y - 1)
	{
		if ((pombuf[(y + 1) * size_x + x] & TRANSISTORS))
		{
			CheckTransistor(x, y + 1);
		}
		else
		{
			if ((pombuf[(y + 1) * size_x + x] & DIFFUSION))
				SetSourceDran(x, y, signals_diff[(y + 1) * size_x + x]);
			else
				otherlen++;
		}
	}
}

void SetupPad(int x, int y, int signalnum, int padtype)
{
	pad tmppad;

	tmppad.x = x;
	tmppad.y = y;
	tmppad.origsignal = signalnum;
	tmppad.type = padtype;
	pads.push_back(tmppad);
	RouteSignal(tmppad.x, tmppad.y, tmppad.origsignal, METAL);
}

// Everything starts here
int _tmain(int argc, _TCHAR* argv[])
{
	__int64 duration = GetTickCount();

	if (argc != 2)
	{
		wprintf(L"Need filename as argument.\n");
		return 0;
	}

	// initializes GDI+ for PNG files manipulation
	_wsetlocale(LC_ALL, L"Czech");
	ULONG_PTR gdiplusToken;
	GdiplusStartupInput gdiplusStartupInput;
	if (GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL) != Ok)
		wprintf(L"Cannot initialize GDI+\n");

	// Loads the layers to pombuffer[]
	CheckFile(argv[1], METAL);
	CheckFile(argv[1], VIAS);
	CheckFile(argv[1], PADS);
	CheckFile(argv[1], POLYSILICON);
	CheckFile(argv[1], DIFFUSION);
	CheckFile(argv[1], BURIED);

	// basic checks of the layers
	for (int y = 0; y < size_y; y++)
		for (int x = 0; x < size_x; x++)
			if ((pombuf[y * size_x + x] & (VIAS | METAL)) == VIAS)
				wprintf(L"Via without metal at: %d %d.\n", x, y);

	for (int y = 0; y < size_y; y++)
		for (int x = 0; x < size_x; x++)
			if ((pombuf[y * size_x + x] & (VIAS | DIFFUSION | POLYSILICON)) == VIAS)
				wprintf(L"Via to nowhere at: %d %d.\n", x, y);

	for (int y = 0; y < size_y; y++)
		for (int x = 0; x < size_x; x++)
			if ((pombuf[y * size_x + x] & (VIAS | METAL | DIFFUSION | POLYSILICON)) == (VIAS | METAL | DIFFUSION | POLYSILICON))
				wprintf(L"Via both to polysilicon and diffusion at: %d %d.\n", x, y);

	for (int y = 0; y < size_y; y++)
		for (int x = 0; x < size_x; x++)
			if ((pombuf[y * size_x + x] & (VIAS | BURIED)) == (VIAS | BURIED))
				wprintf(L"Buried under via at: %d %d.\n", x, y);

	int structure_count = 0;
	type = TRANSISTORS;
	type2 = POLYSILICON | DIFFUSION | BURIED;
	type3 = POLYSILICON | DIFFUSION;
	for (int y = 0; y < size_y; y++)
		for (int x = 0; x < size_x; x++)
			if (shapesize = 0, FillStructure(x, y))
			{
				structure_count++;
				if (shapesize < 49)
					wprintf(L"Transistor at %d, %d is too small.\n", x, y);
			}
	wprintf(L"---------------------\n");
	wprintf(L"Transistor count: %d\n", structure_count);

	structure_count = 0;
	type = VIAS_TO_POLYSILICON;
	type2 = VIAS | POLYSILICON | DIFFUSION;
	type3 = VIAS | POLYSILICON;
	for (int y = 0; y < size_y; y++)
		for (int x = 0; x < size_x; x++)
			if (shapesize = 0, FillStructure(x, y))
			{
				structure_count++;
				if (shapesize < 49)
					wprintf(L"Via to poly at %d, %d is too small.\n", x, y);
			}
	wprintf(L"Vias to poly count: %d\n", structure_count);

	structure_count = 0;
	type = VIAS_TO_DIFFUSION;
	type2 = VIAS | POLYSILICON | DIFFUSION;
	type3 = VIAS | DIFFUSION;
	for (int y = 0; y < size_y; y++)
		for (int x = 0; x < size_x; x++)
			if (shapesize = 0, FillStructure(x, y))
			{
				structure_count++;
				if (shapesize < 49)
					wprintf(L"Via to diffusion at %d, %d is too small.\n", x, y);
			}
	wprintf(L"Vias to diffusion count: %d\n", structure_count);

	structure_count = 0;
	type = BURIED_CONTACT;
	type2 = BURIED | POLYSILICON | DIFFUSION;
	type3 = BURIED | POLYSILICON | DIFFUSION;
	for (int y = 0; y < size_y; y++)
		for (int x = 0; x < size_x; x++)
			if (shapesize = 0, FillStructure(x, y))
			{
				structure_count++;
				if (shapesize < 49)
					wprintf(L"Buried contact at %d, %d is too small.\n", x, y);
			}
	wprintf(L"Buried contacts count: %d\n", structure_count);

	structure_count = 0;
	type = REAL_DIFFUSION;
	type2 = DIFFUSION | TRANSISTORS;
	type3 = DIFFUSION;
	for (int y = 0; y < size_y; y++)
		for (int x = 0; x < size_x; x++)
			if (shapesize = 0, FillStructure(x, y))
			{
				structure_count++;
				if (shapesize < 49)
					wprintf(L"Real diffusion at %d, %d is too small.\n", x, y);
			}
	wprintf(L"Real diffusions count: %d\n", structure_count);

	type = TEMPORARY;
	type2 = BURIED;
	type3 = BURIED_CONTACT;
	for (int y = 0; y < size_y; y++)
		for (int x = 0; x < size_x; x++)
			if (objectfound = false, FillStructureCheck(x, y))
				if (!objectfound)
					wprintf(L"Buried layer without buried contact at: %d %d.\n", x, y);
	
	ClearTemporary();

	signals_metal = new unsigned __int16[size_x * size_y];
	ZeroMemory(&signals_metal[0], size_x * size_y * sizeof(unsigned __int16));
	signals_poly = new unsigned __int16[size_x * size_y];
	ZeroMemory(&signals_poly[0], size_x * size_y * sizeof(unsigned __int16));
	signals_diff = new unsigned __int16[size_x * size_y];
	ZeroMemory(&signals_diff[0], size_x * size_y * sizeof(unsigned __int16));

	// Routes first few named signals
	RouteSignal(150, 800, SIG_GND, METAL);
	RouteSignal(3366, 1370, SIG_VCC, METAL);
//	RouteSignal(6621, 3217, SIG_PHI1, METAL);
//	RouteSignal(6602, 3184, SIG_PHI2, METAL);
//	RouteSignal(6422, 4464, SIG_RESET, METAL);

	// Finds all pads and sets its type

	pad tmppad;

	SetupPad(250, 1500, PAD_PHI2, PAD_INPUT);
	SetupPad(250, 1900, PAD__RST, PAD_INPUT);
	SetupPad(3500, 800, PAD_R_W, PAD_INPUT);

	SetupPad(3500, 500, PAD_D7, PAD_BIDIRECTIONAL);
	SetupPad(3500, 150, PAD_D6, PAD_BIDIRECTIONAL);
	SetupPad(2700, 150, PAD_D5, PAD_BIDIRECTIONAL);
	SetupPad(2200, 150, PAD_D4, PAD_BIDIRECTIONAL);
	SetupPad(1600, 150, PAD_D3, PAD_BIDIRECTIONAL);
	SetupPad(1100, 150, PAD_D2, PAD_BIDIRECTIONAL);
	SetupPad(150, 150, PAD_D1, PAD_BIDIRECTIONAL);
	SetupPad(150, 500, PAD_D0, PAD_BIDIRECTIONAL);

	SetupPad(150, 2300, PAD__CS6, PAD_INPUT);
	SetupPad(750, 2300, PAD__CS5, PAD_INPUT);
	SetupPad(1400, 2300, PAD__CS4, PAD_INPUT);
	SetupPad(2000, 2300, PAD_CS3, PAD_INPUT);
	SetupPad(2600, 2300, PAD_CS2, PAD_INPUT);
	SetupPad(3500, 2300, PAD_CS1, PAD_INPUT);
	SetupPad(3500, 1900, PAD_CS0, PAD_INPUT);

	nextsignal = FIRST_SIGNAL;

	// finds the rest of pads - as all the pads are already numbered there is no pad left
	for (int y = 0; y < size_y; y++)
	{
		for (int x = 0; x < size_x; x++)
		{
			if (!signals_metal[y * size_x + x] && (pombuf[y * size_x + x] & PADS))
			{
				wprintf(L"*** Pad at x:%d y:%d signal:%d\n", x, y, nextsignal);
				RouteSignal(x, y, nextsignal, METAL);
				nextsignal++;
			}
		}
	}
	wprintf(L"---------------------\n");
	wprintf(L"Pads have %d signals.\n", nextsignal - 1);

	// finds the rest of signals starting from METAL then in POLYSILICON and then DIFFUSION
	for (int y = 0; y < size_y; y++)
	{
		for (int x = 0; x < size_x; x++)
		{
			if (!signals_metal[y * size_x + x] && (pombuf[y * size_x + x] & METAL))
			{
				RouteSignal(x, y, nextsignal, METAL);
				nextsignal++;
			}
		}
	}
	wprintf(L"Metal has %d signals.\n", nextsignal - 1);

	for (int y = 0; y < size_y; y++)
	{
		for (int x = 0; x < size_x; x++)
		{
			if (!signals_poly[y * size_x + x] && (pombuf[y * size_x + x] & POLYSILICON))
			{
				RouteSignal(x, y, nextsignal, POLYSILICON);
				nextsignal++;
			}
		}
	}
	wprintf(L"Metal and polysilicon have %d signals.\n", nextsignal - 1);

	for (int y = 0; y < size_y; y++)
	{
		for (int x = 0; x < size_x; x++)
		{
			if (!signals_diff[y * size_x + x] && (pombuf[y * size_x + x] & REAL_DIFFUSION))
			{
				RouteSignal(x, y, nextsignal, DIFFUSION);
				nextsignal++;
			}
		}
	}
	wprintf(L"All together we have %d signals.\n", nextsignal - 1);

	// here the transistors are tested for sanity and put into the vector i.e. list of transistors is built

	int capacitors = 0,
		resistors = 0,
		protecting = 0,
		pulldowns = 0,
		pullups_dep = 0,
		pullups_enh = 0,
		diodes = 0;

	for (int y = 0; y < size_y; y++)
	{
		for (int x = 0; x < size_x; x++)
		{
			if ((pombuf[y * size_x + x] & (TRANSISTORS | TEMPORARY)) == TRANSISTORS)
			{
				gate = source = drain = 0;
				sourcelen = drainlen = otherlen = 0;
				shapesize = 0;
				CheckTransistor(x, y);
				if (!source)
					wprintf(L"Isolated transistor at %d, %d.\n", x, y);
				if (!drain)
				{
					wprintf(L"Capacitor at %d, %d\n", x, y);
					capacitors++;
				}
				if ((source == SIG_VCC) && (drain == SIG_GND))
					wprintf(L"SHORTAGE at %d, %d?\n", x, y);
				if (source == SIG_VCC)
				{
					int tmp = source;
					source = drain;
					drain = tmp;
					tmp = sourcelen;
					sourcelen = drainlen;
					drainlen = tmp;
				}
				if (drain == SIG_GND)
				{
					int tmp = source;
					source = drain;
					drain = tmp;
					tmp = sourcelen;
					sourcelen = drainlen;
					drainlen = tmp;
				}
				if (gate == SIG_GND)
				{
					if (source == SIG_GND)
					{
						protecting++;
					}
					else
					{
						wprintf(L"Transistor always off at %d, %d?\n", x, y);
					}
				}
				else if (gate == SIG_VCC)
				{
					resistors++;
				}
				else
				{
					if (source == SIG_GND)
						pulldowns++;
					if (drain == SIG_VCC)
					{
						if (source == gate)
							pullups_dep++;
						else
							pullups_enh++;
					}
				}
				if ((source != SIG_GND) && (drain != SIG_VCC) && ((gate == source) || (gate == drain)))
				{
					wprintf(L"Diode / resistor at %d, %d.\n", x, y);
					diodes++;
				}
				transistor pomtran;
				pomtran.x = x;
				pomtran.y = y;
				pomtran.area = (float) shapesize;
				// here the area of big transistors is somewhat scaled down to speed the simulation up little bit
				if (pomtran.area > 49.0)
					pomtran.area = (log(pomtran.area / 49.0f) + 1.0f) * 49.0f;
				pomtran.gate = gate;
				pomtran.source = source;
				pomtran.drain = drain;
				pomtran.sourcelen = sourcelen;
				pomtran.drainlen = drainlen;
				pomtran.otherlen = otherlen;
				if ((drain == SIG_VCC) && (source == gate))
					pomtran.depletion = true;
				pomtran.resist = float(otherlen) / (float(sourcelen) + float(drainlen)) /*+ 1.0f*/;
				pomtran.pomchargetogo = QUANTUM / pomtran.resist;
				if (pomtran.pomchargetogo > MAXQUANTUM)
					pomtran.pomchargetogo = MAXQUANTUM;
				transistors.push_back(pomtran);
			}
		}
	}
	ClearTemporary();

	// pull-ups are put on the first positions - well bubble sort is not quite effective...
	for (unsigned int i = 0; i < transistors.size() - 1; i++)
	{
		for (unsigned int j = i + 1; j < transistors.size(); j++)
		{
			if (transistors[i].Valuate() > transistors[j].Valuate())
				std::swap(transistors[i], transistors[j]);
		}
	}

	// finds the connections between transistors and builds the list of transistors connections
	for (unsigned int i = 0; i < transistors.size(); i++)
	{
		for (unsigned int j = 0; j < transistors.size(); j++)
		{
			connection pomcon;
			pomcon.index = j;
			if (transistors[i].gate > SIG_VCC)
			{
				if (transistors[i].gate == transistors[j].gate)
				{
					pomcon.terminal = GATE;
					transistors[i].gateconnections.push_back(pomcon);
					transistors[i].gateneighborhood += transistors[j].area;
				}
				if (transistors[i].gate == transistors[j].source)
				{
					pomcon.terminal = SOURCE;
					transistors[i].gateconnections.push_back(pomcon);
					transistors[i].gateneighborhood += transistors[j].area;
				}
				if (transistors[i].gate == transistors[j].drain)
				{
					pomcon.terminal = DRAIN;
					transistors[i].gateconnections.push_back(pomcon);
					transistors[i].gateneighborhood += transistors[j].area;
				}
			}
			if (transistors[i].source > SIG_VCC)
			{
				if (transistors[i].source == transistors[j].gate)
				{
					pomcon.terminal = GATE;
					transistors[i].sourceconnections.push_back(pomcon);
					transistors[i].sourceneighborhood += transistors[j].area;
				}
				if (transistors[i].source == transistors[j].source)
				{
					pomcon.terminal = SOURCE;
					transistors[i].sourceconnections.push_back(pomcon);
					transistors[i].sourceneighborhood += transistors[j].area;
				}
				if (transistors[i].source == transistors[j].drain)
				{
					pomcon.terminal = DRAIN;
					transistors[i].sourceconnections.push_back(pomcon);
					transistors[i].sourceneighborhood += transistors[j].area;
				}
			}
			if (transistors[i].drain > SIG_VCC)
			{
				if (transistors[i].drain == transistors[j].gate)
				{
					pomcon.terminal = GATE;
					transistors[i].drainconnections.push_back(pomcon);
					transistors[i].drainneighborhood += transistors[j].area;
				}
				if (transistors[i].drain == transistors[j].source)
				{
					pomcon.terminal = SOURCE;
					transistors[i].drainconnections.push_back(pomcon);
					transistors[i].drainneighborhood += transistors[j].area;
				}
				if (transistors[i].drain == transistors[j].drain)
				{
					pomcon.terminal = DRAIN;
					transistors[i].drainconnections.push_back(pomcon);
					transistors[i].drainneighborhood += transistors[j].area;
				}
			}
		}
		for (unsigned int k = 0; k < transistors[i].gateconnections.size(); k++)
			transistors[i].gateconnections[k].proportion = transistors[transistors[i].gateconnections[k].index].area / transistors[i].gateneighborhood;
		for (unsigned int k = 0; k < transistors[i].drainconnections.size(); k++)
			transistors[i].drainconnections[k].proportion = transistors[transistors[i].drainconnections[k].index].area / transistors[i].drainneighborhood;
		for (unsigned int k = 0; k < transistors[i].sourceconnections.size(); k++)
			transistors[i].sourceconnections[k].proportion = transistors[transistors[i].sourceconnections[k].index].area / transistors[i].sourceneighborhood;
	}

	// builds the list of connections of pads
	for (unsigned int i = 0; i < pads.size(); i++)
	{
		for (unsigned int j = 0; j < transistors.size(); j++)
		{
			connection pomcon;
			pomcon.index = j;
			if (pads[i].origsignal > SIG_VCC)
			{
				if (pads[i].origsignal == transistors[j].gate)
				{
					pomcon.terminal = GATE;
					pads[i].connections.push_back(pomcon);
				}
				if (pads[i].origsignal == transistors[j].source)
				{
					pomcon.terminal = SOURCE;
					pads[i].connections.push_back(pomcon);
				}
				if (pads[i].origsignal == transistors[j].drain)
				{
					pomcon.terminal = DRAIN;
					pads[i].connections.push_back(pomcon);
				}
			}
		}
	}

	// builds the list of signals
	for (int i = 0; i < nextsignal; i++)
	{
		signal pomsignal;
		if (i == SIG_VCC || i == SIG_GND)
			pomsignal.ignore = true;
		signals.push_back(pomsignal);
		for (unsigned int j = 0; j < transistors.size(); j++)
		{
			connection pomcon;
			pomcon.index = j;
			if (i == transistors[j].gate)
			{
				pomcon.terminal = GATE;
				signals[i].signalarea += transistors[j].area;
				signals[i].connections.push_back(pomcon);
			}
			if (i == transistors[j].source)
			{
				signals[i].signalarea += transistors[j].area;
				pomcon.terminal = SOURCE;
				signals[i].connections.push_back(pomcon);
			}
			if (i == transistors[j].drain)
			{
				signals[i].signalarea += transistors[j].area;
				pomcon.terminal = DRAIN;
				signals[i].connections.push_back(pomcon);
			}
		}
		for (unsigned int j = 0; j < signals[i].connections.size(); j++)
			signals[i].connections[j].proportion = transistors[signals[i].connections[j].index].area / signals[i].signalarea;
	}

	wprintf(L"---------------------\n");
	wprintf(L"Num of capacitors: %d\n", capacitors);
	wprintf(L"Num of resistors: %d\n", resistors);
	wprintf(L"Num of protecting diodes: %d\n", protecting);
	wprintf(L"Num of depletion pull ups: %d\n", pullups_dep);
	wprintf(L"Num of enhancement pull ups: %d\n", pullups_enh);
	wprintf(L"Num of pull downs: %d\n", pulldowns);
	wprintf(L"Num of diodes: %d\n", diodes);

	wprintf(L"---------------------\n");

	// ================================================================
	// ================================================================
	// Here the list of transistors, pads and signals is ready and done
	// ================================================================
	// =================== now it could be exported ===================
	// ================================================================
	// ================================================================

//	for (unsigned int i = 0; i < pads.size(); i++)
//	{
//		wprintf(L"*** Pad at x:%d y:%d signal:%d cons:%d type:%s\n",
//			pads[i].x, pads[i].y, pads[i].origsignal, pads[i].connections.size(), (pads[i].type == PAD_INPUT ? L"I" : ((pads[i].type == PAD_OUTPUT) ? L"O" : L"B")));
//	}
	
//	for (unsigned int i = 0; i < transistors.size(); i++)
//	{
//		wprintf(L"*** Transistor at x:%d y:%d area:%d gate:%d source:%d drain:%d, sourcelen:%d drainlen:%d otherlen:%d gatecon:%d sourcecon:%d draincon:%d\n",
//			transistors[i].x, transistors[i].y, int(transistors[i].area), transistors[i].gate, transistors[i].source, transistors[i].drain,
//			transistors[i].sourcelen, transistors[i].drainlen, transistors[i].otherlen,
//			transistors[i].gateconnections.size(), transistors[i].sourceconnections.size(), transistors[i].drainconnections.size());
//	}

	// ========================================================================================================================
	// ========================================================================================================================
	// Saves the colored metal layer and vias layer - I commented it out as it did not work properly after some changes in code
	// ========================================================================================================================
	// ========================================================================================================================

	Bitmap *myBitmap = new Bitmap(size_x, size_y);
	BitmapData bitmapdata;
	myBitmap->LockBits(&Rect(0, 0, size_x, size_y), ImageLockModeWrite, PixelFormat32bppARGB, &bitmapdata);
	bd = &bitmapdata;

	for (int y = 0; y < size_y; y++)
	{
		for (int x = 0; x < size_x; x++)
		{
			if (signals_metal[y * size_x + x] == SIG_GND)
				::SetPixelToBitmapData(bd, x, y, 0x0000f);
			else if (signals_metal[y * size_x + x] == SIG_VCC)
				::SetPixelToBitmapData(bd, x, y, 0xff0000);
			else if (pombuf[y * size_x + x] & METAL)
				::SetPixelToBitmapData(bd, x, y, 0x00f00);
			else
				::SetPixelToBitmapData(bd, x, y, 0x000000);
		}
	}

	myBitmap->UnlockBits(&bitmapdata);

	TCHAR filename[256];
	wsprintf(filename, L"%s%s", argv[1], L"_metal_VCC_GND.png");

	CLSID pngClsid;
	GetEncoderClsid(L"image/png", &pngClsid);
	myBitmap->Save(filename, &pngClsid);

	// -------------------------------------------------------

	myBitmap->LockBits(&Rect(0, 0, size_x, size_y), ImageLockModeWrite, PixelFormat32bppARGB, &bitmapdata);
	bd = &bitmapdata;

	for (int y = 0; y < size_y; y++)
	{
		for (int x = 0; x < size_x; x++)
		{
			if (pombuf[y * size_x + x] & VIAS)
			{
				if (signals_metal[y * size_x + x] == SIG_GND)
					::SetPixelToBitmapData(bd, x, y, 0x3f3fff);
				else if (signals_metal[y * size_x + x] == SIG_VCC)
					::SetPixelToBitmapData(bd, x, y, 0xff3f3f);
				else
					::SetPixelToBitmapData(bd, x, y, 0xff7f00);
			}
			else
			{
				::SetPixelToBitmapData(bd, x, y, 0x000000);
			}
		}
	}

	myBitmap->UnlockBits(&bitmapdata);

	wsprintf(filename, L"%s%s", argv[1], L"_vias_VCC_GND.png");
	myBitmap->Save(filename, &pngClsid);
	
	delete myBitmap;
	GdiplusShutdown(gdiplusToken);

	// =============================
	// End of saving colored bitmaps
	// =============================

	delete pombuf;
	delete signals_diff;
	delete signals_poly;
	delete signals_metal;


	// -------------------------------------------------------
	// -------------------------------------------------------
	// -------------------------------------------------------
	// ---------- Here the simulation itself starts ----------
	// -------------------------------------------------------
	// -------------------------------------------------------
	// -------------------------------------------------------

	int lastadr = 0;
	int lastdata = 0;
	int pomadr = 0;
	bool pomale = false;
	bool pom_wr = true;
	bool pom_rd = true;
	bool pom_iom = true;
	bool pom_resetin = true;
	bool pom_inta = true;

	bool pom_s0 = true;
	bool pom_s1 = true;
	int outcounter = 0;
	bool justwasoutput = false;

	bool pom_p1 = false;
	bool pom_p2 = false;

	duration = GetTickCount() - duration;
	wprintf(L"---------------------\n");
	wprintf(L"Duration: %dms\n\n", duration);
	duration = GetTickCount();

	// ======================================================================
	// ============================= Simulation =============================
	// ======================================================================

	wprintf(L"-------------------------------------------------------\n");
	wprintf(L"----------------- Starting simulation -----------------\n");
	wprintf(L"-------------------------------------------------------\n");

	int totcycles = 0;

	// maximally 2000000 iterations
	for (int i = 0; i < 2000000; i++)
	{
		// Setting input pads
		// I commented out several tests like test of READY, SID and HOLD pads
		for (unsigned int j = 0; j < pads.size(); j++)
		{
			if (pads[j].type == PAD_INPUT)
			{
				if (pads[j].origsignal == PAD__RST)
				{
					if (i < DIVISOR * 8)
					{
						pads[j].SetInputSignal(SIG_GND);
						pom_resetin = false;
					}
					else
					{
						pads[j].SetInputSignal(SIG_VCC);
						pom_resetin = true;
					}
				}
				else if (pads[j].origsignal == PAD_PHI2)
				{
					int pom = i / DIVISOR;
					if (pom & 1)
						pads[j].SetInputSignal(SIG_VCC);
					else
						pads[j].SetInputSignal(SIG_GND);
				}
				else if (pads[j].origsignal == PAD_R_W)
				{
					if ((i >= DIVISOR * 10) && (((i / DIVISOR) & 31) == 17))
					{
						pads[j].SetInputSignal(SIG_GND);
						pom_rd = false;
					}
					else
					{
						pads[j].SetInputSignal(SIG_VCC);
						pom_rd = true;
					}
				}
				else if (pads[j].origsignal == PAD_CS0)
				{
					if ((i >= DIVISOR * 10) && (((i / DIVISOR) & 15) == 1))
						pads[j].SetInputSignal(SIG_VCC);
					else
						pads[j].SetInputSignal(SIG_GND);
				}
				else if (pads[j].origsignal == PAD_CS1)
				{
					if ((i >= DIVISOR * 10) && (((i / DIVISOR) & 15) == 1))
						pads[j].SetInputSignal(SIG_VCC);
					else
						pads[j].SetInputSignal(SIG_GND);
				}
				else if (pads[j].origsignal == PAD_CS2)
				{
					if ((i >= DIVISOR * 10) && (((i / DIVISOR) & 15) == 1))
						pads[j].SetInputSignal(SIG_VCC);
					else
						pads[j].SetInputSignal(SIG_GND);
				}
				else if (pads[j].origsignal == PAD_CS3)
				{
					if ((i >= DIVISOR * 10) && (((i / DIVISOR) & 15) == 1))
						pads[j].SetInputSignal(SIG_VCC);
					else
						pads[j].SetInputSignal(SIG_GND);
				}
				else if (pads[j].origsignal == PAD__CS4)
				{
						pads[j].SetInputSignal(SIG_GND);
				}
				else if (pads[j].origsignal == PAD__CS5)
				{
						pads[j].SetInputSignal(SIG_GND);
				}
				else if (pads[j].origsignal == PAD__CS6)
				{
						pads[j].SetInputSignal(SIG_GND);
				}
			}
			else if (pads[j].type == PAD_BIDIRECTIONAL) // we have to pull data bus up or down when memory, I/O or interrupt instruction is read
			{
				{
					if (pom_rd) // nothing is read
					{
						pads[j].SetInputSignal(SIG_FLOATING);
					}
					else
					{
						if (pads[j].origsignal == PAD_D7)
						{
							pads[j].SetInputSignal(SIG_GND);
						}
						else if (pads[j].origsignal == PAD_D6)
						{
							pads[j].SetInputSignal(SIG_GND);
						}
						else if (pads[j].origsignal == PAD_D5)
						{
							pads[j].SetInputSignal(SIG_GND);
						}
						else if (pads[j].origsignal == PAD_D4)
						{
							pads[j].SetInputSignal(SIG_GND);
						}
						else if (pads[j].origsignal == PAD_D3)
						{
							pads[j].SetInputSignal(SIG_GND);
						}
						else if (pads[j].origsignal == PAD_D2)
						{
							pads[j].SetInputSignal(SIG_GND);
						}
						else if (pads[j].origsignal == PAD_D1)
						{
							pads[j].SetInputSignal(SIG_GND);
						}
						else if (pads[j].origsignal == PAD_D0)
						{
							if ((i >= DIVISOR * 10) && (((i / DIVISOR) & 63) == 17 + 32))
								pads[j].SetInputSignal(SIG_VCC);
							else
								pads[j].SetInputSignal(SIG_GND);
						}
					}
				}
			}
			else
			{
			//	some other code here ...
			}
		}
		// End of Setting input pads

		// Simulation itself
		for (unsigned int j = 0; j < transistors.size(); j++)
		{
			transistors[j].Simulate();
		}
		for (unsigned int j = 0; j < signals.size(); j++)
			if (!signals[j].ignore)
				signals[j].Homogenize();
		for (unsigned int j = 0; j < transistors.size(); j++)
			transistors[j].Normalize();
		// End of Simulation itself

		// Reading output pads
		if (!(i % (DIVISOR * 5)))
		{
			wprintf(L"       : P/R          ///    \n", i);
			wprintf(L"       : HR/ DDDDDDDD CCCCCCC\n", i);
			wprintf(L"       : 2SW 76543210 6543210\n", i);
		}


		if (!(i % (DIVISOR / 5))) // writes out every 100s cycle (for output to be not too verbous)
		{
			wprintf(L"%07d: ", i);
			for (unsigned int j = 0; j < pads.size(); j++)
			{
				int pom;
				if (pads[j].type != PAD_INPUT)
				{
					if ((pads[j].type == PAD_BIDIRECTIONAL) && (pads[j].ReadInputStatus() != SIG_FLOATING))
						pom = pads[j].ReadInputStatus();
					else
						pom = pads[j].ReadOutputStatus();
				}
				else
				{
					pom = pads[j].ReadInputStatus();
				}
				char pom2 = pom + '0';
				if (pom == 3)
					pom2 = '.';
				else
					pom2--;
				if (pads[j].origsignal == PAD_PHI2)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD__RST)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD_R_W)
					wprintf(L"%c ", pom2);
				if (pads[j].origsignal == PAD_D7)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD_D6)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD_D5)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD_D4)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD_D3)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD_D2)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD_D1)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD_D0)
					wprintf(L"%c ", pom2);
				if (pads[j].origsignal == PAD__CS6)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD__CS5)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD__CS4)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD_CS3)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD_CS2)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD_CS1)
					wprintf(L"%c", pom2);
				if (pads[j].origsignal == PAD_CS0)
					wprintf(L"%c", pom2);
			//	End of if (pads[j].type == PAD_OUTPUT)
			}
			wprintf(L"\n");
		}
		totcycles = i;
	}

	duration = GetTickCount() - duration;
	wprintf(L"---------------------\n");
	wprintf(L"Duration: %dms\n", duration);
	wprintf(L"Speed of simulation: %.2fHz\n", (double(totcycles) / 2.0) / double(duration) * 1000.0 / double(DIVISOR));

	return 0;
}
